@@ -36,7 +36,6 @@ static ObActionsAct* actions_build_act_from_string(const gchar *name);
 
 static ObActionsAct *interactive_act = NULL;
 static guint         interactive_initial_state = 0;
-static gboolean      replay_pointer = FALSE;
 
 struct _ObActionsDefinition {
     guint ref;
@@ -59,7 +58,6 @@ struct _ObActionsAct {
 
 static GSList *registered = NULL;
 
-
 void actions_startup(gboolean reconfig)
 {
     if (reconfig) return;
@@ -148,7 +146,7 @@ static ObActionsAct* actions_build_act_from_string(const gchar *name)
         actions_definition_ref(act->def);
         act->options = NULL;
     } else
-        g_message(_("Invalid action '%s' requested. No such action exists."),
+        g_message(_("Invalid action \"%s\" requested. No such action exists."),
                   name);
 
     return act;
@@ -224,16 +222,6 @@ static void actions_setup_data(ObActionsData *data,
     data->client = client;
 }
 
-void actions_set_need_pointer_replay_before_move(gboolean replay)
-{
-    replay_pointer = replay;
-}
-
-gboolean actions_get_need_pointer_replay_before_move()
-{
-    return replay_pointer;
-}
-
 void actions_run_acts(GSList *acts,
                       ObUserAction uact,
                       guint state,
@@ -346,14 +334,8 @@ gboolean actions_interactive_input_event(XEvent *e)
 void actions_client_move(ObActionsData *data, gboolean start)
 {
     static gulong ignore_start = 0;
-    if (start) {
+    if (start)
         ignore_start = event_start_ignore_all_enters();
-        if (replay_pointer) {
-            /* replay the pointer event before any windows move */
-            XAllowEvents(ob_display, ReplayPointer, event_curtime);
-            replay_pointer = FALSE;
-        }
-    }
     else if (config_focus_follow &&
              data->context != OB_FRAME_CONTEXT_CLIENT)
     {
@@ -364,8 +346,14 @@ void actions_client_move(ObActionsData *data, gboolean start)
                that moves windows our from under the cursor, the enter
                event will come as a GrabNotify which is ignored, so this
                makes a fake enter event
+
+               don't do this if there is a grab on the pointer.  enter events
+               are ignored during a grab, so don't force fake ones when they
+               should be ignored
             */
-            if ((c = client_under_pointer()) && c != data->client) {
+            if ((c = client_under_pointer()) && c != data->client &&
+                !grab_on_pointer())
+            {
                 ob_debug_type(OB_DEBUG_FOCUS,
                               "Generating fake enter because we did a "
                               "mouse-event action");
