@@ -91,6 +91,7 @@ gint config_mouse_screenedgetime;
 guint    config_menu_hide_delay;
 gboolean config_menu_middle;
 guint    config_submenu_show_delay;
+guint    config_submenu_hide_delay;
 gboolean config_menu_client_list_icons;
 gboolean config_menu_manage_desktops;
 
@@ -812,6 +813,8 @@ static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         config_menu_middle = parse_bool(doc, n);
     if ((n = parse_find_node("submenuShowDelay", node)))
         config_submenu_show_delay = parse_int(doc, n);
+    if ((n = parse_find_node("submenuHideDelay", node)))
+        config_submenu_hide_delay = parse_int(doc, n);
     if ((n = parse_find_node("applicationIcons", node)))
         config_menu_client_list_icons = parse_bool(doc, n);
     if ((n = parse_find_node("manageDesktops", node)))
@@ -1015,7 +1018,8 @@ void config_startup(ObParseInst *i)
 
     config_menu_hide_delay = 250;
     config_menu_middle = FALSE;
-    config_submenu_show_delay = 0;
+    config_submenu_show_delay = 200;
+    config_submenu_hide_delay = 400;
     config_menu_client_list_icons = TRUE;
     config_menu_manage_desktops = TRUE;
     config_menu_files = NULL;
