@@ -53,6 +53,7 @@ void prop_startup(void)
     CREATE(wm_client_leader, "WM_CLIENT_LEADER");
     CREATE(wm_transient_for, "WM_TRANSIENT_FOR");
     CREATE(motif_wm_hints, "_MOTIF_WM_HINTS");
+    CREATE(motif_wm_info, "_MOTIF_WM_INFO");
 
     CREATE(sm_client_id, "SM_CLIENT_ID");
 
@@ -178,6 +179,9 @@ void prop_startup(void)
     CREATE(ob_wm_action_undecorate, "_OB_WM_ACTION_UNDECORATE");
     CREATE(ob_wm_state_undecorated, "_OB_WM_STATE_UNDECORATED");
     CREATE(ob_control, "_OB_CONTROL");
+    CREATE(ob_role, "_OB_ROLE");
+    CREATE(ob_name, "_OB_NAME");
+    CREATE(ob_class, "_OB_CLASS");
 }
 
 #include <X11/Xutil.h>
