@@ -55,6 +55,7 @@ typedef struct Atoms {
     Atom wm_client_leader;
     Atom wm_transient_for;
     Atom motif_wm_hints;
+    Atom motif_wm_info;
 
     /* SM atoms */
     Atom sm_client_id;
@@ -201,6 +202,9 @@ typedef struct Atoms {
     Atom ob_theme;
     Atom ob_config_file;
     Atom ob_control;
+    Atom ob_role;
+    Atom ob_name;
+    Atom ob_class;
 } Atoms;
 extern Atoms prop_atoms;
 
